#!/usr/bin/env python3
# -*-  python-indent:2; tab-width:2; indent-tabs-mode:t; -*-

import operator
import os
import glob
import re
import sys
import logging
import socket
from math import sqrt

import numpy as np
#import scipy as sp
import scipy.stats as sp_stats

def mean_confidence_interval(data, confidence=0.95):
	a = 1.0*np.array(data)
	n = len(a)
	m, se = np.mean(a), sp_stats.sem(a)
	h = se * sp_stats.t._ppf((1+confidence)/2., n-1)
	return h

def ensure_dir(d):
	if not os.path.exists(d):
		os.makedirs(d)
		logging.info("Directory {} created".format(d))

# def mean(a_list):
# 	return sum(a_list,0.0)/len(a_list)

# def sample_standard_dev_of_mean(a_list):
# 	avg        = mean(a_list)
# 	sample_var = sum( [ (x - avg)**2 for x in a_list ], 0.0 ) / (len(a_list)-1)
# 	return sqrt(sample_var/len(a_list))

format_string = '%(asctime)s {} %(levelname)s:%(message)s'.format(socket.gethostname())
logging.basicConfig(format=format_string,
										level=logging.WARNING,
										datefmt='%Y-%m-%d %H:%M:%S')

files = [("death", 0), ("convergence", 1), ("redundancy", 2), ("message", 3)]
graphDir = "graphs"

if len(sys.argv)>1:
	dir_to_plot = sys.argv
else:
	dir_to_plot = glob.glob("results_*")

ensure_dir(graphDir)

# clean

os.system("rm {0}/*.data {0}/*.png  {0}/*.plot {0}/*.pdf".format(graphDir))

#for input in glob.glob("results*.txt"):
for input_dir in (f for f in dir_to_plot if os.path.isdir(f)):
	logging.info("Processing result files of {}".format(input_dir))
	#modifier = input[len("results"):-len(".txt")]
	modifier = input_dir[len("results_"):]

	# FT30Oct13: Only works on "pre-packaged" 
	# results = open(input, 'r').read().split("\n\n\n\n")

	# FT30Oct13: creating paquets

	linesToKeep = ["Noeuds initalise: ",
								 " datapoints died in an epic battle.",
								 "Converge time: ",
								 "Redundancy time: ",
								 "Max Average Message cost: "]

	regexpLinesToKeep = re.compile(".*(" + ('|'.join( [ "("+l+")" for l in linesToKeep ]) + ")") )

	nbLines     = len(linesToKeep)
	results     = [] # list of paquet
	count_files = 0

	# cycling through all result files of directory input_dir
	for input in glob.glob("{0}/expe_results_*.txt".format(input_dir)):
		logging.info("Processing result file {}".format(input))
		count_files+=1
		paquet = [None] * nbLines # current paquet under construction
		count   = 0  # number of lines in current paquet
		for line in open(input, 'r'):
			match = regexpLinesToKeep.match(line)
			if match:
				index_in_paquet = [ i for i in range(2,7) if match.group(i) ][0]-2
				logging.debug("Line is {}".format(line.rstrip()))
				logging.debug("index_in_paquet is {}".format(index_in_paquet))
				line = line.rstrip()
				if (index_in_paquet==0) and (count>0):
					# last paquet not complete, warning + reset
					logging.warning("Incomplete experiment found in {}. Ignored.".format(input))
					paquet = [None] * nbLines # reset paquet and count
					count  = 0
			  # needs to be redone when order of lines get mixed up
				assert paquet[index_in_paquet]==None
				paquet[index_in_paquet] = line; # .append(line);
				count+=1
				if count == nbLines:
					results.append(paquet) # store resulting paquet
					paquet = [None] * nbLines # reset paquet and count
					count  = 0

		if (count!=0):
			logging.warning("Last experiment data incomplete in {}. Ignored.".format(input))

	logging.info("Found {} files and {} runs in {}.".format(count_files,len(results),input_dir))
		
	data    = {}
	average = {}
	interval_confid = {}
	failureTurn = 20


	# Sum the data
	for lines in results:

		# print("durr\n" + paquet)

		nbNoeud     = int(lines[0][len("Noeuds initalise: "):])
		death       = int(lines[1][:-len(" datapoints died in an epic battle.")])
		convergence = int(lines[2][len("Converge tiem: "):]) - failureTurn
		redundancy  = int(lines[3][len("Redundancy tiem: "):]) - failureTurn
		message     = int(lines[4][len("Max Average Message cost: "):].split(' ')[1])
		# print("{0}: {1} {2} {3} {4}"\
			# .format(nbNoeud, death, convergence, redundancy, message))

		tmp = [death, convergence, redundancy, message, 1]
		if nbNoeud in data:
			for i in range(4):
				data[nbNoeud][i].append(tmp[i])
		else:
				data[nbNoeud] = { i:[tmp[i]] for i in range(4)}
		# data[nbNoeud] = [tmp]
			
		# print('\n\n')

	# Divide them
	for nbNoeuds in data:
		average[nbNoeuds] = [ np.mean(data[nbNoeuds][i]) for i in range(4) ]
		interval_confid[nbNoeuds] = [ mean_confidence_interval(data[nbNoeuds][i]) for i in range(4) ]
		# print("{0}: {1} {2} {3} {4}"\
			# .format(item, data[item][0], data[item][1], data[item][2], \
				# data[item][3]))

	results = [ [nbNoeuds,average[nbNoeuds],interval_confid[nbNoeuds]] for nbNoeuds in data]

	# Sort lines, write it to files
	# sortedData = sorted(average.items(), key=operator.itemgetter(0))
	sortedData = sorted(results, key=operator.itemgetter(0))


	for dataType in files:
		dataFile = "{2}/{0}_{1}.data".format(dataType[0], modifier,graphDir)
		logging.info("Writing out {} files.".format(dataFile))
		output = open(dataFile, 'w')

		for line in sortedData:
			output.write("{0} {1} {2}\n".format(line[0], line[1][dataType[1]], line[2][dataType[1]]))

		output.close()

# Run gnuplot on all data plots
logging.info("Producing gnuplot graphs.")

os.chdir(graphDir)

print("* Creating commands file for gnuplot")
commands = open('graphs.plot', 'w')

font = "Times New Roman"
# font = "DejaVu Sans Mono"

commands.write("""set terminal  pdf size 5,3 font '{0},12'
#set term png size 1024,1024
set palette gray
set key right bottom
set xlabel "#nodes"
set style fill solid 0.3
#set size 0.6, 0.6
set pointsize 2.5
set grid ytics lt 0 lw 1 lc rgb "#bbbbbb"
set grid xtics lt 0 lw 1 lc rgb "#bbbbbb\n""".format(font))

#colors = ["royalblue", "goldenrod", "red", "forest-green", "grey0"]
colors = ["#0266C8", "#F2B50F", "#F90101", "#00933B", "#000000"]

for dataType in files:
	commands.write('set output "{0}.pdf"\n'.format(dataType[0]))
	commands.write('set ylabel "{0}"\n'.format(dataType[0], font))
	# commands.write('set title "{0}"\n'.format(dataType[0].capitalize()))
	commands.write('set title ""\n')

	curves = []

	i = 0
	inputs = glob.glob("{0}*.data".format(dataType[0]))
	inputs.sort()	# that way, the curve's order will be the same in every graph
	for input in inputs:
		title = input[:len(input)-len(".data")]

		curves.append('"{0}" with errorlines lw 2.5 linecolor rgb "{2}" title "{1}" , '\
				.format(input, title, colors[i]))
		# tmp += '"{0}" with errorlines title "{1}", '\
				# .format(input, title)

		i = (i + 1)%len(colors) # at least the script doesn't crash

	curves.reverse()

	tmp = "plot [:] [0:] " + "".join(curves)
	tmp = tmp[:-2] + '\n'

	commands.write(tmp)

commands.close()


# commands.write('set term png size 1024,1024\n')
# # commands.write('set term pdf\n')
# for dataType in files:
# 	tmp = 'set output "{0}.png"\nplot '.format(dataType[0])
# 	for data in glob.glob("{0}*.data".format(dataType[0])):
# 		tmp += '"{0}" with errorlines, '.format(data)

# 	tmp = tmp[:-2] + '\n'
# 	commands.write(tmp)

# commands.close()


# Run GnuPlot
print("* Running gnuplot")
os.system("gnuplot graphs.plot")
